
# q-expansion of newform 945.2.cj.b, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 945
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[596, [-1, 0, 1, 0]], [757, [0, 0, 0, 1]], [136, [0, 0, 1, 0]]]
aps_data = [[-1, -1, 0, 1], [0, 0, 0, 0], [1, 0, 0, -2], [2, 0, -3, 0], [2, 0, 0, 0], [3, 3, -1, -2], [0, 0, 4, 4], [0, -1, -1, -1], [1, 4, -4, -1], [8, -3, -4, 3], [-4, 2, 2, -2], [4, -5, 1, 1], [0, -3, 0, 0], [1, 2, 1, -1], [-4, 4, 5, 1], [0, 0, 2, 2], [0, -7, -3, -7], [4, -4, 4, 0], [-4, 1, 3, 3], [-3, -2, 0, 1], [4, 4, -8, 4], [-2, 8, -2, 0], [9, -4, -5, -5], [0, 4, 8, 4], [10, 7, -3, 3], [1, 0, -2, -2], [-5, -1, -1, -5], [1, 7, -8, -8], [1, -8, 1, 0], [-4, 4, -4, -8], [11, -1, -1, 11], [-4, 0, 8, -12], [-1, 0, 0, -1], [-3, 9, 3, -18], [-2, 0, 4, 12], [-9, -10, 0, 5], [-6, -5, 1, -1], [-6, 9, 15, -15], [3, -3, 1, 4], [-16, -16, 16, 0], [1, -3, 1, 0], [7, 0, -14, -8], [-15, 3, 15, -6], [-6, 2, 8, -8], [-4, 0, 0, -4], [-6, 6, 6, -12], [-20, 4, 20, -8], [1, 4, -5, -5], [7, 18, -18, -7], [14, -10, 0, 5], [-14, -4, 10, -10], [0, 2, 0, 0], [-2, 0, 4, 19], [5, 0, -10, 5], [-2, 6, -6, 2], [-5, 3, -3, 5], [-18, -6, 18, 12], [-18, -17, 9, 17], [0, -6, -6, 0], [0, -1, 24, -1], [-1, 11, -10, -10], [2, 2, 6, -8], [-5, -1, 1, 5], [14, -5, -7, 5], [-6, -6, 18, -12], [7, 7, -3, -4], [6, 0, -6, 0], [-1, -1, -11, 12], [1, -8, 7, 7], [0, 0, 10, 0], [7, 4, 4, 7], [-8, -4, 4, 4], [5, -3, 3, -5], [-7, 4, -4, 7], [13, 0, -26, 5], [-10, 13, 13, -10], [-11, 0, 22, -10], [4, 20, 16, -16], [-1, 12, 0, -6], [0, 10, 5, 10], [11, -5, -11, 10], [0, 3, -10, 3], [5, 9, -5, -18], [19, -14, -14, 19], [22, 8, -22, -16], [10, -10, -9, 1], [5, 0, -10, 16], [15, 15, 7, -22], [-30, 2, 15, -2], [10, -10, 5, 15], [5, 0, -5, 5], [19, -2, 0, 1], [-13, -13, -11, 24], [-1, -9, 1, 18], [6, 0, -12, 28], [-3, -1, -1, -3], [0, 38, 0, -19], [-5, 16, -5, 0], [11, 2, -13, -13], [0, 4, 0, 4], [-23, 3, 20, 20], [-20, -20, -8, 28], [0, 3, -3, -3], [-4, -4, -4, 0], [0, -3, -1, -3], [6, -6, 16, 22], [25, 10, -15, 15], [-6, -13, 19, 19], [16, -32, -8, 32], [16, 20, -8, -20], [6, -9, 9, -6], [-3, 3, 13, 10], [-12, -12, 20, -8], [-20, -24, 0, 12], [2, 16, 0, -8], [-28, -12, 0, 6], [-25, -25, 20, 5], [3, -3, 14, 17], [-12, 2, -2, 12], [-10, -31, 5, 31], [12, 13, -6, -13], [-8, -7, 1, -1], [1, -1, 25, 26], [23, -23, 0, 23], [-2, 16, -2, 0], [8, 22, 0, -11], [20, -6, 20, 0], [0, 4, -16, 4], [4, -13, -17, 17], [3, 20, 20, 3], [-12, -20, -12, 0], [23, -23, -16, 7], [37, 14, 0, -7], [-10, 12, 12, -10], [3, 0, -6, 4], [-20, 3, 20, -6], [23, 23, 3, -26], [5, 36, 31, -31], [-27, 27, 31, 4], [-10, 27, -10, 0], [9, 0, -18, -9], [26, -9, -26, 18], [-7, -17, -10, 10], [-16, -5, -5, -16], [-27, 10, 27, -20], [0, 29, 5, 29], [-10, 14, -4, -4], [-2, -8, 8, 2], [19, -10, 0, 5], [-4, 29, 33, -33], [-1, -6, -6, -1], [26, 0, -52, -6], [22, -7, 7, -22], [1, 15, -15, -1], [8, 1, 1, 8], [0, -1, 3, -1], [18, 3, -9, -3], [11, 12, -11, -24], [7, -4, 4, -7], [22, -12, -11, 12], [5, 5, -5, 0], [-14, -22, 22, 14], [-33, -33, 21, 12], [-2, 17, 1, -17], [6, 20, -26, -26], [-36, 11, 11, -36], [8, 0, -8, 0], [33, 6, -6, -33]]
